/* -*- C -*-
 * Copyright (c) 1999 Allan Saddi <asaddi@philosophysw.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: list.c,v 1.5 1999/03/17 19:03:43 asaddi Exp $
 */

#include <stdlib.h>

#include <assert.h>

#define NO_PSW_LIST_INLINE
#include "list.h"

#ifndef lint
static const char rcsid[] =
	"$Id: list.c,v 1.5 1999/03/17 19:03:43 asaddi Exp $";
#endif /* !lint */

void
PREFIX(InitList) (struct PREFIX(List) *list)
{
  assert (list != NULL);

  list->head = (struct PREFIX(Node) *) &list->dummy;
  list->dummy = NULL;
  list->tail = (struct PREFIX(Node) *) &list->head;
}

void
PREFIX(InsertBefore) (struct PREFIX(Node) *before, struct PREFIX(Node) *node)
{
  assert (before != NULL);
  assert (node != NULL);

  node->next = before;
  node->prev = before->prev;

  before->prev->next = node;
  before->prev = node;
}

void
PREFIX(AddHead) (struct PREFIX(List) *list, struct PREFIX(Node) *node)
{
  assert (list != NULL);
  assert (node != NULL);

  PREFIX(InsertBefore) (list->head, node);
}

void
PREFIX(AddTail) (struct PREFIX(List) *list, struct PREFIX(Node) *node)
{
  assert (list != NULL);
  assert (node != NULL);

  PREFIX(InsertBefore) ((struct PREFIX(Node) *) &list->dummy, node);
}

void
PREFIX(Remove) (struct PREFIX(Node) *node)
{
  assert (node != NULL);

  node->prev->next = node->next;
  node->next->prev = node->prev;
}

void
PREFIX(InsertSorted) (struct PREFIX(List) *list, struct PREFIX(Node) *node,
		      int (*cmpfunc) (struct PREFIX(Node) *,
				      struct PREFIX(Node) *))
{
  struct PREFIX(Node) *before;

  assert (list != NULL);
  assert (node != NULL);
  assert (cmpfunc != NULL);

  FOR_EACH_NODE_IN_LIST (list, before)
    if (cmpfunc (node, before) < 0)
      break;

  PREFIX(InsertBefore) (before, node);
}

struct PREFIX(Node) *
PREFIX(RemHead) (struct PREFIX(List) *list)
{
  struct PREFIX(Node) *node;

  assert (list != NULL);

  if (list->head->next) {
    node = list->head;
    PREFIX(Remove) (list->head);
    return node;
  }
  else
    return NULL;
}

struct PREFIX(Node) *
PREFIX(RemTail) (struct PREFIX(List) *list)
{
  struct PREFIX(Node) *node;

  assert (list != NULL);

  if (list->head->next) {
    node = list->tail;
    PREFIX(Remove) (list->tail);
    return node;
  }
  else
    return NULL;
}
