/* -*- C -*-
 * Copyright (c) 1999 Allan Saddi <asaddi@philosophysw.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: list.h,v 1.3 1999/01/19 08:13:23 asaddi Exp $
 */

#ifndef _PSW_LIST_H
#define _PSW_LIST_H

#include <stdlib.h>

#include <assert.h>

#ifndef PREFIX
#define PREFIX(name) name
#endif /* PREFIX */

struct PREFIX(Node) {
  struct PREFIX(Node) *next;
  struct PREFIX(Node) *prev;
};

struct PREFIX(List) {
  struct PREFIX(Node) *head;
  struct PREFIX(Node) *dummy;
  struct PREFIX(Node) *tail;
};

#define FOR_EACH_NODE_IN_LIST(list,node) \
  for ((node) = (void *) (list)->head; \
       ((struct PREFIX(Node) *) (node))->next; \
       (node) = (void *) ((struct PREFIX(Node) *) (node))->next)

#define IS_LIST_EMPTY(list) (!(list)->head->next)

#ifdef NO_PSW_LIST_INLINE
void PREFIX(InitList) (struct PREFIX(List) *list);
void PREFIX(InsertBefore) (struct PREFIX(Node) *before,
			   struct PREFIX(Node) *node);
void PREFIX(AddHead) (struct PREFIX(List) *list, struct PREFIX(Node) *node);
void PREFIX(AddTail) (struct PREFIX(List) *list, struct PREFIX(Node) *node);
void PREFIX(Remove) (struct PREFIX(Node) *node);
#else /* !NO_PSW_LIST_INLINE */
extern inline void PREFIX(InitList) (struct PREFIX(List) *list) {
  assert (list != NULL);

  list->head = (struct PREFIX(Node) *) &list->dummy;
  list->dummy = NULL;
  list->tail = (struct PREFIX(Node) *) &list->head;
}

extern inline void PREFIX(InsertBefore) (struct PREFIX(Node) *before,
					 struct PREFIX(Node) *node) {
  assert (before != NULL);
  assert (node != NULL);

  node->next = before;
  node->prev = before->prev;

  before->prev->next = node;
  before->prev = node;
}

extern inline void PREFIX(AddHead) (struct PREFIX(List) *list,
				    struct PREFIX(Node) *node) {
  assert (list != NULL);
  assert (node != NULL);

  PREFIX(InsertBefore) (list->head, node);
}

extern inline void PREFIX(AddTail) (struct PREFIX(List) *list,
				    struct PREFIX(Node) *node) {
  assert (list != NULL);
  assert (node != NULL);

  PREFIX(InsertBefore) ((struct PREFIX(Node) *) &list->dummy, node);
}

extern inline void PREFIX(Remove) (struct PREFIX(Node) *node) {
  assert (node != NULL);

  node->prev->next = node->next;
  node->next->prev = node->prev;
}
#endif /* NO_PSW_LIST_INLINE */

void PREFIX(InsertSorted) (struct PREFIX(List) *list,
			   struct PREFIX(Node) *node,
			   int (*cmpfunc) (struct PREFIX(Node) *,
					   struct PREFIX(Node) *));
struct PREFIX(Node) *PREFIX(RemHead) (struct PREFIX(List) *list);
struct PREFIX(Node) *PREFIX(RemTail) (struct PREFIX(List) *list);

#endif /* _PSW_LIST_H */
