/* -*- C -*-
 * Copyright (c) 1999 Allan Saddi <asaddi@philosophysw.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: newsfish.h,v 1.13 2000/08/02 05:43:39 asaddi Exp $
 */

#ifndef NEWSFISH_H
#define NEWSFISH_H

#include <sys/types.h>
#include <limits.h>
#include <regex.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "list.h"

#define PROGRAM_NAME "newsfish"

#define MAX_SERVER_NAME 256

struct Server {
  struct Node node;
  char name[MAX_SERVER_NAME]; /* Name of the server. */
  int port;

  /* Any of these fields may be NULL to indicate that the default
     should be used. */
  char *host; /* Actual name of host. If NULL, assume same as name. */
  char *rcFile; /* Name of the rc file. */
  char *newsRCFile; /* Name of the newsrc file. */
  char *password; /* Server password */
  char *user; /* Server user */
  char *preconnectCmd; /* Pre-connect command or NULL */
  char *postconnectCmd; /* Post-connect command or NULL */
};

#define MAX_NEWSGROUP_NAME 256

enum WildcardType {
  WILD_TYPE_NONE,
  WILD_TYPE_ALL,
  WILD_TYPE_PREFIX,
  WILD_TYPE_SUFFIX,
  WILD_TYPE_MIDDLE,

  WILD_TYPE_MAX
};

/* State information for a newsgroup. */
struct Newsgroup {
  struct Node node;
  char name[MAX_NEWSGROUP_NAME]; /* Name of the newsgroup. */
  int lastArticle; /* The *next* article to be read. */
  struct List actionList; /* List of filter sets. See below. */
  enum WildcardType wildType; /* Wildcard type. */
};

#define ACTION_TYPE_NONE -1
#define ACTION_TYPE_FILE 0
#define ACTION_TYPE_PIPE 1

/* Each newsgroup can have multiple sets of filters. */
struct Action {
  struct Node node;
  struct List conditionList; /* List of Conditions. See below. */
  int type; /* What type of action this is (save to file, pipe, etc.) */
  char file[PATH_MAX]; /* The file to dump matching articles to. */
  unsigned long sizeLimit; /* The file size limit. */
  int match; /* Per-filter set match flag for matching. */
};

/* A match condition. */
struct Condition {
  struct Node node;
  regex_t regexp; /* The regexp, compiled by regcomp() */
  int invert; /* The sense of the regexp should be inverted. */
};

extern int BeVerbose;
extern unsigned long GlobalSizeLimit;
extern int DefaultPort;
extern int KeepGoing;

void *mymalloc (size_t size);
char *mystrdup (const char *str);
FILE *myfopen (const char *path, const char *mode);

int ProcessAction (struct Action *actInfo,
		   const char *header, int headerLen,
		   const char *body, int bodyLen);

int MatchWildNewsgroup (const char *newsgroup, struct Newsgroup *ngInfo);

#endif /* NEWSFISH_H */
